/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2024 Mobileye Vision Technologies Ltd.
 */

#ifndef _DT_BINDINGS_CLOCK_MOBILEYE_EYEQ5_CLK_H
#define _DT_BINDINGS_CLOCK_MOBILEYE_EYEQ5_CLK_H

#define EQ5C_PLL_CPU	0
#define EQ5C_PLL_VMP	1
#define EQ5C_PLL_PMA	2
#define EQ5C_PLL_VDI	3
#define EQ5C_PLL_DDR0	4
#define EQ5C_PLL_PCI	5
#define EQ5C_PLL_PER	6
#define EQ5C_PLL_PMAC	7
#define EQ5C_PLL_MPC	8
#define EQ5C_PLL_DDR1	9

#define EQ5C_DIV_OSPI	10

#endif
