/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_C8SECTPFE_H
#define __DT_C8SECTPFE_H

#define STV0367_TDA18212_NIMA_1	0
#define STV0367_TDA18212_NIMA_2	1
#define STV0367_TDA18212_NIMB_1	2
#define STV0367_TDA18212_NIMB_2	3

#define STV0903_6110_LNB24_NIMA	4
#define STV0903_6110_LNB24_NIMB	5

#endif /* __DT_C8SECTPFE_H */
