/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides macros for Palmas device bindings.
 *
 * Copyright (c) 2013, NVIDIA Corporation.
 *
 * Author: Laxman Dewangan <ldewangan@nvidia.com>
 *
 */

#ifndef __DT_BINDINGS_PALMAS_H
#define __DT_BINDINGS_PALMAS_H

/* External control pins */
#define PALMAS_EXT_CONTROL_PIN_ENABLE1	1
#define PALMAS_EXT_CONTROL_PIN_ENABLE2	2
#define PALMAS_EXT_CONTROL_PIN_NSLEEP	3

#endif /* __DT_BINDINGS_PALMAS_H */
