/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2016-2018, The Linux Foundation. All rights reserved.
 */

#ifndef __DT_QCOM_RPMH_RSC_H__
#define __DT_QCOM_RPMH_RSC_H__

#define SLEEP_TCS	0
#define WAKE_TCS	1
#define ACTIVE_TCS	2
#define CONTROL_TCS	3

#endif /* __DT_QCOM_RPMH_RSC_H__ */
